'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _ms = require('ms');

var _ms2 = _interopRequireDefault(_ms);

var _httpErrors = require('http-errors');

var _httpErrors2 = _interopRequireDefault(_httpErrors);

var _shortid = require('shortid');

var _shortid2 = _interopRequireDefault(_shortid);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _lodash = require('lodash');

var _pubsub = require('../pubsub');

var _constants = require('../constants');

var _constants2 = _interopRequireDefault(_constants);

var _proxyLink = require('./proxy-link');

var _proxyLink2 = _interopRequireDefault(_proxyLink);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PROXY_REQUEST = _constants2.default.PROXY_REQUEST;
var PROXY_OFFER = _constants2.default.PROXY_OFFER;
var PROXY_LINK = _constants2.default.PROXY_LINK;
var PROXY_LINK_DROP = _constants2.default.PROXY_LINK_DROP;
var PROXY_DELIVER = _constants2.default.PROXY_DELIVER;


var PROXY_REQUEST_INTERVAL = (0, _ms2.default)('1s');
var PROXY_REQUEST_TIMEOUT = (0, _ms2.default)('20s');
var PROXY_HEARTBEAT_INTERVAL = (0, _ms2.default)('1s');
var PROXY_HEARTBEAT_TTL = (0, _ms2.default)('5s');

var defaultRedisConfig = {
  host: '127.0.0.1',
  port: 6379
};

var Client = function () {
  function Client() {
    var config = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
    (0, _classCallCheck3.default)(this, Client);

    (0, _assert2.default)((0, _lodash.isObject)(config), 'Invalid config');

    this.proxy = null;
    this.pubsub = null;
    this.started = false;
    this.id = _shortid2.default.generate();

    this.debug = (0, _debug2.default)('tlaloc:client:' + this.id);

    this.requestInterval = config.requestInterval || PROXY_REQUEST_INTERVAL;
    this.requestTimeout = config.requestTimeout || PROXY_REQUEST_TIMEOUT;
    this.heartbeatInterval = config.heartbeatInterval || PROXY_HEARTBEAT_INTERVAL;
    this.heartbeatTTL = config.heartbeatTTL || PROXY_HEARTBEAT_TTL;

    // bind methods to self
    this.emitProxyRequest = this.emitProxyRequest.bind(this);
    this.onProxyOffer = this.onProxyOffer.bind(this);
    this.onProxyLink = this.onProxyLink.bind(this);
    this.onProxyLinkDrop = this.onProxyLinkDrop.bind(this);

    this.debug('Creating new client');
  }

  (0, _createClass3.default)(Client, [{
    key: 'connect',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
        var redisConfig = arguments.length <= 0 || arguments[0] === undefined ? defaultRedisConfig : arguments[0];
        var host, port;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                (0, _assert2.default)((0, _lodash.isObject)(redisConfig), 'Invalid redis config');
                (0, _assert2.default)(!this.started, 'Client already started');

                host = redisConfig.host;
                port = redisConfig.port;

                this.debug('Connecting to ' + host + ':' + port);
                this.started = true;

                _context.next = 8;
                return (0, _pubsub.createRedisPubSub)({ port: port, host: host, id: this.id });

              case 8:
                this.pubsub = _context.sent;


                this.debug('PubSub connected');

              case 10:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function connect(_x2) {
        return _ref.apply(this, arguments);
      }

      return connect;
    }()
  }, {
    key: 'getProxy',
    value: function getProxy() {
      var _this = this;

      return new _promise2.default(function (resolve, reject) {
        (0, _assert2.default)(_this.pubsub, 'Redis PubSub has not been started');
        _this.proxyResolver = resolve;
        _this.proxyReject = reject;

        _this.requestProxy();
      });
    }
  }, {
    key: 'requestProxy',
    value: function requestProxy() {
      var _this2 = this;

      this.debug('Requesting proxy');

      this.pubsub.onceMessage(PROXY_OFFER, this.onProxyOffer);

      this.proxyRequesterInterval = setInterval(this.emitProxyRequest, this.requestInterval);

      this.proxyRequesterTimeout = setTimeout(function () {
        _this2.debug('Proxy request timeout');
        _this2.removeHandlers();
        _this2.proxyReject((0, _httpErrors2.default)(408));
      }, this.requestTimeout);

      this.emitProxyRequest();
    }
  }, {
    key: 'emitProxyRequest',
    value: function emitProxyRequest() {
      this.pubsub.send(PROXY_REQUEST);
    }
  }, {
    key: 'onProxyOffer',
    value: function onProxyOffer(_ref2) {
      var clientId = _ref2.clientId;

      this.debug('Proxy offered by ' + clientId);
      this.removeHandlers();
      this.debug('Requesting proxy link');
      this.pubsub.sendMessage(PROXY_LINK, clientId);
      this.pubsub.onceMessage(PROXY_LINK_DROP, this.onProxyLinkDrop);
      this.pubsub.onceMessage(PROXY_DELIVER, this.onProxyLink);
    }
  }, {
    key: 'onProxyLink',
    value: function onProxyLink(data) {
      this.debug('Proxy link from', data);
      this.removeHandlers();

      this.proxy = new _proxyLink2.default({
        pubsub: this.pubsub,
        clientId: data.clientId,
        endpoint: data.payload,
        heartbeatInterval: this.heartbeatInterval,
        heartbeatTTL: this.heartbeatTTL
      });

      var proxyResolver = this.proxyResolver;
      this.proxyResolver = null;
      proxyResolver(this.proxy);
    }
  }, {
    key: 'onProxyLinkDrop',
    value: function onProxyLinkDrop() {
      this.debug('Proxy link request dropped');
      this.removeHandlers();
      this.requestProxy();
    }
  }, {
    key: 'removeHandlers',
    value: function removeHandlers() {
      this.debug('Removing handlers');
      this.pubsub.offMessage(PROXY_OFFER, this.onProxyOffer);
      this.pubsub.offMessage(PROXY_LINK, this.onProxyLink);
      this.pubsub.offMessage(PROXY_LINK_DROP, this.onProxyLinkDrop);
      clearInterval(this.proxyRequesterInterval);
      clearTimeout(this.proxyRequesterTimeout);
    }
  }, {
    key: 'destroy',
    value: function destroy() {
      this.debug('Destroying client');
      this.removeHandlers();
      this.proxyResolver = null;
      this.proxyReject = null;
      this.pubsub.removeAllListeners();
      this.pubsub.removeAllMessageListeners();
      this.pubsub = null;

      if (this.proxy) {
        this.proxy.stop();
        this.proxy = null;
      }
    }
  }]);
  return Client;
}();

exports.default = Client;