'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _events = require('events');

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _constants = require('../constants');

var _constants2 = _interopRequireDefault(_constants);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PROXY_HEARTBEAT = _constants2.default.PROXY_HEARTBEAT;
var PROXY_TIMEOUT = _constants2.default.PROXY_TIMEOUT;
var PROXY_DROP = _constants2.default.PROXY_DROP;

/**
 * Holds the proxy information
 * emits heatbeats to the provider using the client to send events
 * listen to providers heartbeats and kill proxy on timeout
 */

var ProxyLink = function (_EventEmitter) {
  (0, _inherits3.default)(ProxyLink, _EventEmitter);

  function ProxyLink(_ref) {
    var pubsub = _ref.pubsub;
    var clientId = _ref.clientId;
    var endpoint = _ref.endpoint;
    var heartbeatInterval = _ref.heartbeatInterval;
    var heartbeatTTL = _ref.heartbeatTTL;
    (0, _classCallCheck3.default)(this, ProxyLink);

    (0, _assert2.default)(pubsub && clientId && endpoint);

    var parts = endpoint.split(':');

    var _this = (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(ProxyLink).call(this));

    _this.endpoint = endpoint;
    _this.host = parts[0];
    _this.port = parts[1];

    _this.active = true;
    _this.targetClientId = clientId;
    _this.pubsub = pubsub;

    _this.debug = (0, _debug2.default)('tlaloc:proxyLink:' + pubsub.id);
    _this.emitHeartbeat = _this.emitHeartbeat.bind(_this);
    _this.onHeartbeat = _this.onHeartbeat.bind(_this);
    _this.onTimeout = _this.onTimeout.bind(_this);

    _this.heartbeatIntervalMs = heartbeatInterval;
    _this.heartbeatTTLMs = heartbeatTTL;

    _this.heartbeatInterval = setInterval(_this.emitHeartbeat, _this.heartbeatIntervalMs);
    _this.TTLTimeoutId = setTimeout(_this.onTimeout, _this.heartbeatTTLMs);

    _this.pubsub.onMessage(PROXY_HEARTBEAT, _this.onHeartbeat);
    _this.pubsub.onMessage(PROXY_DROP, _this.onProxyDrop);

    _this.debug('created proxy link ' + endpoint + ' from ' + clientId);
    return _this;
  }

  (0, _createClass3.default)(ProxyLink, [{
    key: 'emitHeartbeat',
    value: function emitHeartbeat() {
      this.debug('Heartbeat');
      this.pubsub.sendMessage(PROXY_HEARTBEAT, this.targetClientId);
    }
  }, {
    key: 'onHeartbeat',
    value: function onHeartbeat(_ref2) {
      var clientId = _ref2.clientId;

      if (this.targetClientId === clientId) {
        this.debug('Received heartbeat');
        clearTimeout(this.TTLTimeoutId);
        this.TTLTimeoutId = setTimeout(this.onTimeout, this.heartbeatTTLMs);
      }
    }
  }, {
    key: 'onTimeout',
    value: function onTimeout() {
      this.debug('Connection with the provider lost');
      this.emit(PROXY_TIMEOUT);
      this.stop();
    }
  }, {
    key: 'onProxyDrop',
    value: function onProxyDrop() {
      this.debug('Provider closed proxy link');
      this.emit(PROXY_TIMEOUT);
      this.stop();
    }
  }, {
    key: 'unlink',
    value: function unlink() {
      this.debug('Unlinking proxy');
      this.pubsub.sendMessage(PROXY_DROP, this.targetClientId);
      this.stop();
    }
  }, {
    key: 'stop',
    value: function stop() {
      this.debug('killing proxy link');
      clearInterval(this.heartbeatInterval);
      clearTimeout(this.TTLTimeoutId);
      this.removeAllListeners();
      this.active = false;
      this.host = null;
      this.port = null;
      this.endpoint = null;
      this.pubsub.offMessage(PROXY_HEARTBEAT, this.onHeartbeat);
      this.pubsub = null;
    }
  }]);
  return ProxyLink;
}(_events.EventEmitter);

exports.default = ProxyLink;