'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _lodash = require('lodash');

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _thsInstance = require('./ths-instance');

var _thsInstance2 = _interopRequireDefault(_thsInstance);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = require('debug')('tlaloc:cluster');

/**
 * Creates a pool of tor hidden service instances.
 *
 * @param  {String}  dataDir  The root directory to be used for instance data
 */

var THSCluster = function () {
  function THSCluster(params) {
    (0, _classCallCheck3.default)(this, THSCluster);

    (0, _assert2.default)((0, _lodash.isObject)(params), 'Invalid params');
    (0, _assert2.default)((0, _lodash.isString)(params.dataDir), 'Data dir is not a string');
    (0, _assert2.default)((0, _lodash.isObject)(params.redis), 'Invalid redis config');

    (0, _lodash.defaults)(params, {
      torInstances: 2,
      portRangeStart: 10770,
      host: '127.0.0.1'
    });

    this.config = params;

    this.status = {
      starting: false,
      disconnecting: false,
      ready: false
    };

    this.serviceRotationIndex = 0;

    debug('Creating cluster', this.config);

    this.instances = (0, _lodash.times)(params.torInstances, function (i) {
      var _context;

      var port = parseInt(params.portRangeStart, 10) + i;

      return new _thsInstance2.default({
        dataDir: params.dataDir + '/' + port,
        port: port,
        host: params.host,
        ctrlPort: port + parseInt(params.torInstances, 10),
        redis: params.redis,
        onTorError: (_context = console).error.bind(_context),
        onTorMessage: (_context = console).log.bind(_context)
      });
    });
  }

  /**
   * Starts tor, connects all the hidden services in the pool
   * @return {Promise}
   */


  (0, _createClass3.default)(THSCluster, [{
    key: 'connect',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
        var status;
        return _regenerator2.default.wrap(function _callee$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                status = this.status;

                (0, _assert2.default)(!status.disconnecting, 'Cluster is disconnecting');
                (0, _assert2.default)(!status.starting && !status.ready, 'Cluster was already connected');

                status.starting = true;

                debug('Connecting Tor processes...');

                _context2.next = 7;
                return _promise2.default.all(this.instances.map(function (ths) {
                  return ths.connect();
                }));

              case 7:

                status.starting = false;
                status.ready = true;

                debug('Tor connected with ' + this.instances.length + ' instances');

              case 10:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee, this);
      }));

      function connect() {
        return _ref.apply(this, arguments);
      }

      return connect;
    }()
  }, {
    key: 'disconnect',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
        var status;
        return _regenerator2.default.wrap(function _callee2$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                status = this.status;

                (0, _assert2.default)(!status.starting, 'Can not disconnect while connection is in progress');
                (0, _assert2.default)(!status.disconnecting, 'Cluster already disconnecting');
                (0, _assert2.default)(status.ready, 'Cluster was not started');

                status.disconnecting = true;
                status.ready = false;

                _context3.next = 8;
                return _promise2.default.all(this.instances.map(function (ths) {
                  return ths.disconnect();
                }));

              case 8:

                status.disconnecting = false;

                debug('Tor disconnected');

              case 10:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee2, this);
      }));

      function disconnect() {
        return _ref2.apply(this, arguments);
      }

      return disconnect;
    }()

    /**
     * Returns the running tor services in the cluster.
     * @return {Array}  Tor services running in the cluster.
     */

  }, {
    key: 'getService',


    /**
     * Gets a difference service every time this method is invoked.
     * @return {Object}  A tor hidden service definition.
     */
    value: function getService() {
      this.serviceRotationIndex++;

      var services = this.services;
      var port = this.config.portRangeStart + this.serviceRotationIndex;
      var service = (0, _lodash.find)(services, { name: 'port_' + port });

      if (!service) {
        this.serviceRotationIndex = 0;
        service = services[0];
      }

      return service;
    }
  }, {
    key: 'getPort',
    value: function getPort() {
      var service = this.getService();

      if (!service || !service.ports.length) {
        return null;
      }

      return parseInt(service.ports[0], 10);
    }
  }, {
    key: 'services',
    get: function get() {
      var list = (0, _lodash.reduce)(this.instances, function (list, ths) {
        if (ths.isTorRunning()) {
          var services = ths.getServices();
          if (services.length) {
            list.push(services[0]);
          }
        }
        return list;
      }, []);

      debug('Getting services', list);

      return list;
    }
  }]);
  return THSCluster;
}();

exports.default = THSCluster;