'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _ms = require('ms');

var _ms2 = _interopRequireDefault(_ms);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _constants = require('../constants');

var _constants2 = _interopRequireDefault(_constants);

var _pubsub = require('../pubsub');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PROXY_REQUEST = _constants2.default.PROXY_REQUEST;
var PROXY_OFFER = _constants2.default.PROXY_OFFER;
var PROXY_LINK = _constants2.default.PROXY_LINK;
var PROXY_LINK_DROP = _constants2.default.PROXY_LINK_DROP;
var PROXY_DROP = _constants2.default.PROXY_DROP;
var PROXY_DELIVER = _constants2.default.PROXY_DELIVER;
var PROXY_TIMEOUT = _constants2.default.PROXY_TIMEOUT;
var PROXY_HEARTBEAT = _constants2.default.PROXY_HEARTBEAT;


var PROXY_HEARTBEAT_TTL = (0, _ms2.default)('5s');
var assertNotEmpty = function assertNotEmpty() {
  return new Error('Value not provided');
};

var ProxySupply = function () {
  function ProxySupply(_ref) {
    var heartbeatTTL = _ref.heartbeatTTL;
    var _ref$endpoint = _ref.endpoint;
    var endpoint = _ref$endpoint === undefined ? assertNotEmpty() : _ref$endpoint;
    (0, _classCallCheck3.default)(this, ProxySupply);

    this.endpoint = endpoint;
    this.pubsub = null;
    this.available = false;
    this.linkedTo = null;
    this.heartbeatTTL = heartbeatTTL || PROXY_HEARTBEAT_TTL;
    this.onProxyRequest = this.onProxyRequest.bind(this);
    this.onProxyLink = this.onProxyLink.bind(this);
    this.onProxyUnlink = this.onProxyUnlink.bind(this);
    this.onHeartbeat = this.onHeartbeat.bind(this);
    this.onTimeout = this.onTimeout.bind(this);
    this.unlink = this.unlink.bind(this);
    this.debug = (0, _debug2.default)('tlaloc:cluster:proxySupply:' + this.endpoint);
  }

  (0, _createClass3.default)(ProxySupply, [{
    key: 'connect',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(_ref3) {
        var host = _ref3.host;
        var port = _ref3.port;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                (0, _assert2.default)(!this.pubsub, 'Can not reconnect proxy supply');
                this.debug('Connecting redis pubsub');
                _context.next = 4;
                return (0, _pubsub.createRedisPubSub)({ port: port, host: host, id: this.endpoint });

              case 4:
                this.pubsub = _context.sent;

                this.available = true;
                this.attachListeners();

              case 7:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function connect(_x) {
        return _ref2.apply(this, arguments);
      }

      return connect;
    }()
  }, {
    key: 'disconnect',
    value: function disconnect() {
      clearTimeout(this.TTLTimeoutId);
      this.pubsub.removeAllListeners();
      this.pubsub.removeAllMessageListeners();
      this.pubsub = null;
      this.linkedTo = null;
      this.available = false;
    }
  }, {
    key: 'attachListeners',
    value: function attachListeners() {
      this.debug('Attaching listeners');
      this.pubsub.on(PROXY_REQUEST, this.onProxyRequest);
      this.pubsub.on(PROXY_DROP, this.onProxyUnlink);
      this.pubsub.onMessage(PROXY_LINK, this.onProxyLink);
    }
  }, {
    key: 'onProxyRequest',
    value: function onProxyRequest(_ref4) {
      var clientId = _ref4.clientId;

      if (this.available) {
        this.debug('Sending proxy offer to ' + clientId);
        this.pubsub.sendMessage(PROXY_OFFER, clientId);
      }
    }
  }, {
    key: 'onProxyLink',
    value: function onProxyLink(_ref5) {
      var clientId = _ref5.clientId;

      if (this.available) {
        this.debug('Linking proxy with ' + clientId);
        this.available = false;
        this.linkedTo = clientId;
        this.pubsub.sendMessage(PROXY_DELIVER, clientId, this.endpoint);

        this.pubsub.onMessage(PROXY_HEARTBEAT, this.onHeartbeat);
        this.TTLTimeoutId = setTimeout(this.onTimeout, this.heartbeatTTL);

        // wait for heartbeats
        // if no heartbeat, drop
      } else {
        this.debug('Dropping proxy link with ' + clientId);
        this.pubsub.sendMessage(PROXY_LINK_DROP, clientId);
      }
    }
  }, {
    key: 'onProxyUnlink',
    value: function onProxyUnlink(_ref6) {
      var clientId = _ref6.clientId;

      if (this.linkedTo === clientId) {
        this.debug('Removing proxy link');
        this.unlink();
      }
    }
  }, {
    key: 'onHeartbeat',
    value: function onHeartbeat(_ref7) {
      var clientId = _ref7.clientId;

      if (this.linkedTo === clientId) {
        this.debug('Received heartbeat');
        this.pubsub.sendMessage(PROXY_HEARTBEAT, clientId);
        clearTimeout(this.TTLTimeoutId);
        this.TTLTimeoutId = setTimeout(this.onTimeout, this.heartbeatTTL);
      }
    }
  }, {
    key: 'onTimeout',
    value: function onTimeout() {
      this.pubsub.emit(PROXY_TIMEOUT);
      this.unlink();
    }
  }, {
    key: 'unlink',
    value: function unlink() {
      clearTimeout(this.TTLTimeoutId);
      this.linkedTo = null;
      this.available = true;
    }
  }]);
  return ProxySupply;
}();

exports.default = ProxySupply;