'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createRedisPubSub = undefined;

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _redisPubsubEmitter = require('redis-pubsub-emitter');

var _redisPubsubEmitter2 = _interopRequireDefault(_redisPubsubEmitter);

var _httpErrors = require('http-errors');

var _httpErrors2 = _interopRequireDefault(_httpErrors);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _nodeUuid = require('node-uuid');

var _nodeUuid2 = _interopRequireDefault(_nodeUuid);

var _lodash = require('lodash');

var _ms = require('ms');

var _ms2 = _interopRequireDefault(_ms);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = require('debug')('tlaloc:pubsub');

var prototype = {

  onMessageHandlers: [],

  // Broadcast message to the network
  send: function send(event, data) {
    (0, _assert2.default)(event);
    debug('broadcasting "' + event + '"');
    var message = this.createMessage(data);
    this.publish(event, message);
  },


  // Send message to an specific addressee
  sendMessage: function sendMessage(event, to, data) {
    (0, _assert2.default)(event);
    debug('sendMessage "' + event + '" to ' + to);
    var message = this.createMessage(data, to);
    this.publish(event, message);
  },


  // Append your identity to the message and the addressee if included
  createMessage: function createMessage(data, to) {
    var message = {
      clientId: this.id,
      payload: data || {}
    };

    if (to) {
      message.to = to;
    }

    return message;
  },
  onMessage: function onMessage(event, fn) {
    (0, _assert2.default)(event && fn);
    debug('register onMessage "' + event + '"');

    var wrappedFn = function wrappedFn(data) {
      debug('data', data);
      if (data.to !== this.id) return;
      fn.apply(this, arguments);
    };

    wrappedFn = wrappedFn.bind(this);

    this.onMessageHandlers.push([event, fn, wrappedFn]);
    this.on(event, wrappedFn);
  },


  // Register a function that is executed only once if its addressed to you
  onceMessage: function onceMessage(event, fn) {
    (0, _assert2.default)(event && fn);
    debug('register onceMessage "' + event + '"');
    var _wrappedFn = function wrappedFn(data) {
      if (data.to !== this.id) return;
      this.offMessage(event, fn);
      this.removeListener(event, _wrappedFn);
      fn.apply(this, arguments);
    };

    _wrappedFn = _wrappedFn.bind(this);

    this.onMessageHandlers.push([event, fn, _wrappedFn]);
    this.on(event, _wrappedFn);
  },


  // Remove function of the
  offMessage: function offMessage(event, fn) {
    (0, _assert2.default)(event && fn);
    var toRemove = [];
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = (0, _getIterator3.default)(this.onMessageHandlers), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var arr = _step.value;

        var _arr = (0, _slicedToArray3.default)(arr, 3);

        var _event = _arr[0];
        var func = _arr[1];
        var wrappedFn = _arr[2];

        if (fn === func) {
          toRemove.push(arr);
          this.removeListener(_event, wrappedFn);
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  },
  removeAllMessageListeners: function removeAllMessageListeners() {
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      for (var _iterator2 = (0, _getIterator3.default)(this.onMessageHandlers), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        var _step2$value = (0, _slicedToArray3.default)(_step2.value, 3);

        var event = _step2$value[0];
        var wrappedFn = _step2$value[2];

        this.removeListener(event, wrappedFn);
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }

    this.onMessageHandlers.length = 0;
  }
};

/**
* Extends the redisPubSub and adds 5s timeout
*/
var createRedisPubSub = exports.createRedisPubSub = function createRedisPubSub(_ref) {
  var port = _ref.port;
  var host = _ref.host;
  var _ref$id = _ref.id;
  var id = _ref$id === undefined ? _nodeUuid2.default.v4() : _ref$id;

  (0, _assert2.default)(port && host && id, 'Invalid params');
  (0, _assert2.default)((0, _lodash.isString)(id), 'Client ID must be a string');

  return new _promise2.default(function (resolve, reject) {
    var pubsub = (0, _assign2.default)(_redisPubsubEmitter2.default.createClient(port, host), prototype, { id: id });

    pubsub.setMaxListeners(100);

    var timeout = setTimeout(function () {
      pubsub.removeListener('ready', onReady); // eslint-disable-line
      reject((0, _httpErrors2.default)(504, 'Redis connection timeout'));
    }, (0, _ms2.default)('5s'));

    var onReady = function onReady() {
      clearTimeout(timeout);
      debug('connected to redis');
      resolve(pubsub);
    };

    pubsub.once('ready', onReady);
  });
};

exports.default = { createRedisPubSub: createRedisPubSub };